@@ -1,10 +1,10 @@
 ;;; kirigami.el --- A unified method to fold and unfold text -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2025 James Cherti | https://www.jamescherti.com/contact/
+;; Copyright (C) 2025-2026 James Cherti | https://www.jamescherti.com/contact/
 
 ;; Author: James Cherti
-;; Package-Version: 20251202.1126
-;; Package-Revision: b23cae12111d
+;; Package-Version: 20260122.1320
+;; Package-Revision: d6126f95e019
 ;; URL: https://github.com/jamescherti/kirigami.el
 ;; Keywords: convenience
 ;; Package-Requires: ((emacs "26.3"))
@@ -110,13 +110,25 @@ specific reason to disable these enhancements."
   :group 'kirigami)
 
 (defvar kirigami-fold-list
-  `(((outline-indent-mode)
-     :open-all   outline-indent-open-folds
-     :close-all  outline-indent-close-folds
-     :toggle     outline-indent-toggle-fold
-     :open       outline-indent-open-fold
-     :open-rec   outline-indent-open-fold-rec
-     :close      outline-indent-close-fold)
+  `(((outline-indent-minor-mode)
+     :open-all   ,(lambda ()
+                    (when (fboundp 'outline-indent-open-folds)
+                      (call-interactively 'outline-indent-open-folds)))
+     :close-all  ,(lambda ()
+                    (when (fboundp 'outline-indent-close-folds)
+                      (call-interactively 'outline-indent-close-folds)))
+     :toggle     ,(lambda ()
+                    (when (fboundp 'outline-indent-toggle-fold)
+                      (call-interactively 'outline-indent-toggle-fold)))
+     :open       ,(lambda ()
+                    (when (fboundp 'outline-indent-open-fold)
+                      (call-interactively 'outline-indent-open-fold)))
+     :open-rec   ,(lambda ()
+                    (when (fboundp 'outline-indent-open-fold-rec)
+                      (call-interactively 'outline-indent-open-fold-rec)))
+     :close      ,(lambda ()
+                    (when (fboundp 'outline-indent-close-fold)
+                      (call-interactively 'outline-indent-close-fold))))
     ((vdiff-mode)
      :open-all   vdiff-open-all-folds
      :close-all  vdiff-close-all-folds
@@ -191,9 +203,7 @@ specific reason to disable these enhancements."
       markdown-mode
       gfm-mode)
      :open-all   show-all
-     :close-all  ,(lambda ()
-                    (when (fboundp 'hide-sublevels)
-                      (hide-sublevels 1)))
+     :close-all  kirigami--outline-close-all
      :toggle     outline-toggle-children
      :open       ,(lambda ()
                     (cond
@@ -211,17 +221,16 @@ specific reason to disable these enhancements."
                         (ignore-errors
                           (show-entry)
                           (show-children))))))
-     :open-rec   show-subtree
-     :close      ;; hide-subtree
-     ,(lambda ()
-        (cond
-         ((and kirigami-enhance-outline
-               (fboundp 'kirigami--outline-hide-subtree))
-          (kirigami--outline-hide-subtree))
+     :open-rec show-subtree
+     :close ,(lambda ()
+               (cond
+                ((and kirigami-enhance-outline
+                      (fboundp 'kirigami--outline-hide-subtree))
+                 (kirigami--outline-hide-subtree))
 
-         (t
-          (when (fboundp 'hide-subtree)
-            (hide-subtree))))))
+                (t
+                 (when (fboundp 'hide-subtree)
+                   (hide-subtree))))))
     ;; TODO support vimish-fold-mode
     ;; ((vimish-fold-mode)
     ;;  :open-all   ,(lambda () (call-interactively 'vimish-fold-unfold-all))
@@ -304,7 +313,7 @@ would ignore `:close-all' actions and invoke the provided functions on
           (let* ((actions (cdar list))
                  (fn (plist-get actions action)))
             (when fn
-              (with-demoted-errors "Error: %S" (funcall fn))))
+              fn))
         (kirigami-fold--action-get-func (cdr list) action ignore-errors)))))
 
 (defun kirigami-fold-action (list action &optional ignore-errors)
@@ -364,6 +373,30 @@ This is the Emacs version of `outline-hide-subtree'."
         (outline-flag-subtree t))
     (error "Required outline functions are undefined")))
 
+(defun kirigami--outline-close-all ()
+  "Close all folds and ensure the first heading remains visible."
+  (when (fboundp 'hide-sublevels)
+    (hide-sublevels 1)
+
+    (when (and (fboundp 'outline-on-heading-p)
+               (fboundp 'outline-back-to-heading))
+      (let ((heading-point (save-excursion
+                             (condition-case nil
+                                 (progn
+                                   (goto-char (window-start))
+                                   (beginning-of-visual-line)
+                                   (if (outline-on-heading-p)
+                                       (point)
+                                     (outline-back-to-heading t)))
+                               (error
+                                nil)))))
+        ;; Ensure folded headings remain visible after hiding subtrees. Fixes a
+        ;; bug in outline and Evil where headings could scroll out of view when
+        ;; their subtrees were folded. TODO Send a patch to Emacs and/or Evil
+        (when (and heading-point
+                   (< heading-point (window-start)))
+          (set-window-start (selected-window) heading-point t))))))
+
 (defun kirigami--outline-show-entry (&rest _)
   "Ensure the current heading and body are fully visible.
 Repeatedly reveal children and body until the entry is no longer folded.
@@ -404,25 +437,45 @@ the entry is fully visible."
     (error "Required outline functions are undefined")))
 
 (defun kirigami--outline-hide-subtree ()
-  "Close the previous lower-level heading if current heading is folded or empty.
-If the current heading is folded or empty, move to the previous heading
-with a lower level and close its subtree. Otherwise, close the current subtree."
+  "Close the current heading's subtree in a robust manner.
+
+If the current heading is folded or contains no content, move to the previous
+heading with a higher level and close its subtree.
+
+Otherwise, close the current subtree. Ensures that folded headings remain
+visible in the window after hiding."
   (if (and (fboundp 'outline-back-to-heading)
            (fboundp 'outline-end-of-subtree)
            (fboundp 'outline-up-heading)
            (fboundp 'outline-on-heading-p))
       (save-excursion
+        ;; Move to the current heading; error if before the first heading
         (outline-back-to-heading)
-        (if (or (kirigami--outline-heading-folded-p)
-                ;; Fold without any content
-                (let ((start (save-excursion (end-of-line) (point)))
-                      (end (save-excursion (outline-end-of-subtree) (point))))
-                  (= start end)))
-            (progn
-              (outline-up-heading 1 t)
-              (when (outline-on-heading-p)
-                (kirigami--outline-legacy-hide-subtree)))
-          (kirigami--outline-legacy-hide-subtree)))
+
+        (let ((heading-point (point)))
+          ;; If the current heading is folded, or if it contains no content,
+          ;; move to the previous higher-level heading.
+          (when (or (kirigami--outline-heading-folded-p)  ; Folded?
+                    ;; Or fold without any content
+                    (let ((start (save-excursion (end-of-line)
+                                                 (point)))
+                          (end (save-excursion (outline-end-of-subtree)
+                                               (point))))
+                      (= start end)))
+            ;; Try to move up to previous higher-level heading
+            (outline-up-heading 1 t)
+            (setq heading-point (point)))
+
+          (when (outline-on-heading-p)
+            (kirigami--outline-legacy-hide-subtree))
+
+          ;; Ensure folded headings remain visible after hiding subtrees. Fixes
+          ;; a bug in outline and Evil where headings could scroll out of view
+          ;; when their subtrees were folded.
+          ;; TODO Send a patch to Emacs and/or Evil
+          (when (and heading-point
+                     (< heading-point (window-start)))
+            (set-window-start (selected-window) heading-point t))))
     (error "Required outline functions are undefined")))
 
 ;;; Functions: open/close folds
